{ $Id: PDOConnectFirebird.pas 52 2007-01-30 00:10:24Z jmarino $

############################################################################################

	Pascal Data Objects Library
	Copyright  2006 John Marino, http://www.synsport.com
	Project site: http://pdo.sourceforge.net

############################################################################################

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

############################################################################################}


unit PDOConnectFirebird;

interface

{$I ../../directives.inc}

uses
  Types, Classes, SysUtils,
  PDOInterfaces,
  PDOConnection,
  PDOStatement,
  PDOClasses,
  PDOConstantsFirebird,
  PDOFirebirdDriver,
  PDOLogging,
  PDOUtilsFirebird,
  PDOStatementFirebird,
  PDOMessages;

type
  TFirebirdConnect = class(TAbstractConnection)
  private
    FProtocol: TFirebirdBaseDriver;
    FProtocolDescription: AnsiString;
    FDialect: Word;
    FHandle: TISC_DB_HANDLE;
    FTrHandle: TISC_TR_HANDLE;
    FStatusVector: TARRAY_ISC_STATUS;
    FInfo: TStringList;
    FLastUpdateCount: integer;

    function CreateNewDatabase(const SQL: ansistring): shortInt;
    procedure StartTransaction;

  public


    constructor Create(parameters: TStringList; protocol: TFirebirdBaseDriver);
    destructor Destroy; override;

    function InitializeStatement: IPDOStatement; override;

    procedure Commit; override;
    procedure Rollback; override;

    procedure Open; override;
    procedure Close; override;


    function GetProtocol: TFirebirdBaseDriver;
    function execute_without_result (SQL: AnsiString): Int64; override;
    function GetConnectionHandle: PCONNECTION; override;
    function GetSQLDialect: TSQLDialect; override;
    function GetDescription: AnsiString; override;
  protected
    function GetSilentMode: Byte; override;
    function GetCaseMode: Byte; override;
    function GetOracleNull: Byte; override;
    function GetBufferQuery: Boolean; override;
    function GetMaximumBLOBSize: Int64; override;
    function GetCompression: Boolean; override;
    function GetMultipleQuery: Boolean; override;

    procedure SetSilentMode (mode: Byte); override;
    procedure SetCaseMode (mode: Byte);   override;
    procedure SetOracleNull (mode: Byte);  override;
    procedure setBufferQuery (mode: Boolean); override;
    procedure setMaximumBLOBSize (BLOBSize: Int64); override;
    procedure setCompression (mode: Boolean); override;
    procedure setMultipleQuery (mode: Boolean); override;


    function GetLastInsertID: Int64; override;
    function GetSQLState: AnsiString; override;
    function GetLastErrorCode: Integer; override;
    function GetLastErrorMessage: AnsiString; override;
    function GetServerVersion: AnsiString; override;
    function GetClientVersion: AnsiString; override;
    function GetServerInfo: AnsiString;  override;
    function GetClientInfo: AnsiString;  override;
  end;

implementation


{**
  Constructs this object and assignes the main properties.
  @param Driver the parent PDO driver.
  @param PlainDriver a MySQL plain driver.
  @param HostName a name of the host.
  @param Port a port number (0 for default port).
  @param Database a name pof the database.
  @param User a user name.
  @param Password a user password.
  @param Info a string list with extra connection parameters.
}
constructor TFirebirdConnect.Create(parameters: TStringList; protocol: TFirebirdBaseDriver);
var
  RoleName: ansistring;
  ClientCodePage: ansistring;
  UserSetDialect: ansistring;
  trunc_password: ansistring;
begin

    inherited Create(
        parameters.Values['datasource'],
        strtoint(parameters.Values['port']),
        parameters.Values['database'],
        parameters.Values['username'],
        parameters.Values['password'],
        'no_socket');


    self.FProtocol := protocol;
    self.FProtocolDescription := self.FProtocol.GetProtocol;

    if self.FProtocolDescription = 'interbase-5' then
        FDialect := 1
    else
        FDialect := 3;

    UserSetDialect := Trim(parameters.Values['dialect']);
    if UserSetDialect <> '' then
        FDialect := StrToIntDef(UserSetDialect, FDialect);

    ClientCodePage := Trim(parameters.Values['codepage']);

    if ClientCodePage <> '' then
        parameters.Add('isc_dpb_lc_ctype=' + UpperCase(ClientCodePage));

    RoleName := Trim(parameters.Values['rolename']);
    if RoleName <> '' then
        parameters.Add('isc_dpb_sql_role_name='+  UpperCase(RoleName));

    parameters.Add('isc_dpb_username=' + parameters.Values['username']);

    trunc_password := parameters.Values['password'];
    if length(trunc_password) > 8 then trunc_password := copy (trunc_password, 1, 8);
    parameters.Add('isc_dpb_password=' +  trunc_password);


    self.FInfo := parameters;
    self.FLastUpdateCount := -1;
    self.Open;

end;

{**
  Destroys this object and cleanups the memory.
}
destructor TFirebirdConnect.Destroy;
begin
    if not self.Closed then self.Close;
    inherited Destroy;

end;




{**
  Opens a connection to database server with specified parameters.
}
procedure TFirebirdConnect.Open;
var
  DPB: PChar;
  FDPBLength: Word;
  DBName: array[0..512] of Char;
begin
    if not Closed then Exit;

    DPB := PDOUtilsFirebird.GenerateDPB(self.FInfo, FDPBLength, FDialect);

    if self.HostName <> '' then
        begin
            if Port <> FIREBIRD_DEFAULT_PORT then
                StrPCopy(DBName, self.HostName + '/' + IntToStr(Port) + ':' + self.Database)
            else
                StrPCopy(DBName, self.HostName + ':' + self.Database)
        end
    else
        StrPCopy(DBName, self.Database);

    try
        self.FHandle := nil;
        self.FProtocol.isc_attach_database(@FStatusVector, StrLen(DBName), DBName, @FHandle, FDPBLength, DPB);

        PDOUtilsFirebird.CheckFirebirdError(self.FProtocol, self.FStatusVector, lcConnect);
        if assigned (self.FHandle) then
        begin
            self.FProtocol.log(lcConnect, Format('CONNECT TO "%s" AS USER "%s"', [Database, User]));


            { To support autocommit mode, we always start a transaction }
            StartTransaction;

            inherited Open;
        end;
  finally
    StrDispose(DPB);
  end;

end;

{**
   Commit current transaction
}
procedure TFirebirdConnect.Commit;
Begin
  if FTrHandle <> nil then
  begin
    self.FProtocol.isc_commit_retaining(@FStatusVector, @FTrHandle);
    PDOUtilsFirebird.CheckFirebirdError(self.FProtocol, self.FStatusVector, lcTransaction);
    if not self.AutoCommit then
        self.FProtocol.log(lcTransaction, BEndTransaction);
  end;
end;



{**
  Drops all changes made since the previous
  commit/rollback and releases any database locks currently held
  by this Connection. This method should be used only when auto-
  commit has been disabled.
  @see #setAutoCommit
}
procedure TFirebirdConnect.Rollback;
begin
  if FTrHandle <> nil then
  begin
    self.FProtocol.isc_rollback_retaining(@FStatusVector, @FTrHandle);
    PDOUtilsFirebird.CheckFirebirdError(self.FProtocol, self.FStatusVector, lcTransaction);
    self.FProtocol.log(lcTransaction, SRollbackSuccessful);
  end;
end;


procedure TFirebirdConnect.Close;
begin
  if Closed then Exit;

  if FTrHandle <> nil then
  begin
    if self.AutoCommit then
        begin
          self.FProtocol.isc_commit_transaction(@FStatusVector, @FTrHandle);
          self.FProtocol.log(lcExecute, format (SQuerySuccessful, [self.Database]));
        end
    else
        begin
          self.FProtocol.isc_rollback_transaction(@FStatusVector, @FTrHandle);
          self.FProtocol.log(lcTransaction, SRollbackSuccessful + ' - ' + self.Database);
        end;
    FTrHandle := nil;
    PDOUtilsFirebird.CheckFirebirdError(self.FProtocol, self.FStatusVector, lcDisconnect);
  end;

  if FHandle <> nil then
  begin
    self.FProtocol.isc_detach_database(@FStatusVector, @FHandle);
    FHandle := nil;
    PDOUtilsFirebird.CheckFirebirdError(self.FProtocol, self.FStatusVector, lcDisconnect);
  end;

  self.FProtocol.log(lcConnect, Format('DISCONNECT FROM "%s"', [Database]));

  inherited Close;
end;




{**
  Gets a reference to Firebird connection handle.
  @return a reference to Firebird connection handle.
}
function TFirebirdConnect.GetConnectionHandle: PCONNECTION;
begin
  Result := @FHandle;
end;

{**
  Gets a Firebird plain driver interface.
  @return a Firebird plain driver interface.
}
function TFirebirdConnect.GetProtocol : TFirebirdBaseDriver;
begin
  Result := FProtocol;
end;


function TFirebirdConnect.GetSilentMode: Byte;
Begin
    Result := self.FProtocol.GetSilentMode;
End;

function TFirebirdConnect.GetCaseMode: Byte;
Begin
    Result := self.FProtocol.GetCaseMode;
End;

function TFirebirdConnect.GetOracleNull: Byte;
Begin
    Result := self.FProtocol.GetOracleNull;
End;

function TFirebirdConnect.GetBufferQuery: Boolean;
Begin
    Result := self.FProtocol.GetBufferQuery;
End;

function TFirebirdConnect.GetMaximumBLOBSize: Int64;
Begin
    Result := self.FProtocol.GetMaximumBLOBSize;
End;

function TFirebirdConnect.GetSQLDialect: TSQLDialect;
Begin
    Result := self.FProtocol.GetSQLDialect;
End;

function TFirebirdConnect.GetLastInsertID: Int64;
Begin
    Result := self.FProtocol.getLastInsertID(self.FHandle);
End;

function TFirebirdConnect.GetSQLState: AnsiString;
Begin
    Result := self.FProtocol.getSQLState;
End;

function TFirebirdConnect.GetLastErrorCode: Integer;
Begin
    Result := self.FProtocol.getLastErrorCode;
End;

function TFirebirdConnect.GetLastErrorMessage: AnsiString;
Begin
    Result := self.FProtocol.getLastErrorMessage;
End;

function TFirebirdConnect.GetServerVersion: AnsiString;
Begin
    Result := self.FProtocol.GetServerVersion(self.FHandle);
End;

function TFirebirdConnect.GetServerInfo: AnsiString;
Begin
    Result := self.FProtocol.GetServerInfo(self.FHandle);
End;

function TFirebirdConnect.GetClientVersion: AnsiString;
Begin
    Result := self.FProtocol.GetClientVersion;
End;

function TFirebirdConnect.GetClientInfo: AnsiString;
Begin
    Result := self.FProtocol.GetClientInfo;
End;

procedure TFirebirdConnect.SetSilentMode(mode: Byte);
begin
    self.FProtocol.SetSilentMode(mode);
end;

procedure TFirebirdConnect.SetCaseMode(mode: Byte);
begin
    self.FProtocol.SetCaseMode(mode);
end;

procedure TFirebirdConnect.SetOracleNull(mode: Byte);
begin
    self.FProtocol.SetOracleNull(mode);
end;

procedure TFirebirdConnect.setBufferQuery(mode: Boolean);
begin
    self.FProtocol.setBufferQuery(mode);
end;

procedure TFirebirdConnect.setMaximumBLOBSize(BLOBSize: Int64);
begin
    self.FProtocol.setMaximumBLOBSize(BLOBSize);
end;

function TFirebirdConnect.GetDescription: AnsiString;
begin
    Result := self.FProtocol.GetDescription;
end;

function TFirebirdConnect.GetCompression: Boolean;
begin
    Result := self.FProtocol.GetCompression;
end;

procedure TFirebirdConnect.setCompression(mode: Boolean);
begin
    self.FProtocol.setCompression(mode);
end;

function TFirebirdConnect.GetMultipleQuery: Boolean;
begin
    Result := self.FProtocol.GetMultipleQuery;
end;

procedure TFirebirdConnect.setMultipleQuery (mode: Boolean);
begin
    self.FProtocol.setMultipleQuery(mode);
end;


procedure TFirebirdConnect.StartTransaction;
var
  Params: TStringList;
  PTEB: PISC_TEB;
begin
  PTEB := nil;
  Params := TStringList.Create;

  { Set transaction parameters by TransactIsolationLevel }

  {JRM: It looks like the only two values that transactIsolationLevel can be is
    tiNone and tiReadCommitted.  It doesn't look like PDO has built-in support
    to change these levels yet.}


  Params.Add('isc_tpb_version3');
  case self.TransactIsolationLevel of
    tiReadCommitted:
      begin
        Params.Add('isc_tpb_read_committed');
        Params.Add('isc_tpb_rec_version');
        Params.Add('isc_tpb_nowait');
      end;
    tiRepeatableRead:
      begin
        Params.Add('isc_tpb_concurrency');
        Params.Add('isc_tpb_nowait');
      end;
    tiSerializable:
      begin
        Params.Add('isc_tpb_consistency');
      end;
  else
    begin
      { Add user defined parameters for traansaction }
      Params.Clear;
      Params.AddStrings(self.FInfo);
    end;
  end;

  try
    { GenerateTPB return PTEB with null pointer tpb_address from defaul
      transaction }

    PTEB :=  PDOUtilsFirebird.GenerateTPB(Params, self.FHandle);
    self.FProtocol.isc_start_multiple(@FStatusVector, @FTrHandle, 1, PTEB);
    PDOUtilsFirebird.CheckFirebirdError(self.FProtocol, self.FStatusVector, lcTransaction);
    if not self.AutoCommit then
        self.FProtocol.log(lcTransaction, SBeginTransaction);

  finally
    Params.Free;
    StrDispose(PTEB.tpb_address);
    FreeMem(PTEB);
  end
end;

{**
  Creates new database
  @param SQL a sql strinf for creation database
}
function TFirebirdConnect.CreateNewDatabase(const SQL: ansistring): shortInt;
var
  DbHandle: PISC_DB_HANDLE;
  TrHandle: PISC_TR_HANDLE;
begin
  Result := -1;
  DbHandle := nil;
  TrHandle := nil;
  self.FProtocol.isc_dsql_execute_immediate(@FStatusVector, @DbHandle, @TrHandle, 0, PChar(SQL), FDialect, nil);
  PDOUtilsFirebird.CheckFirebirdError(self.FProtocol, FStatusVector, lcExecute, SQL);
  if (self.FProtocol.last_error_code = 0) then result := 1;


 // self.FProtocol.isc_detach_database(@FStatusVector, @DbHandle);
 // PDOUtilsFirebird.CheckFirebirdError(self.FProtocol, FStatusVector, lcExecute, SQL);
end;




function TFirebirdConnect.execute_without_result (SQL: AnsiString): Int64;
var
  StmtHandle: TISC_STMT_HANDLE;
  StatementType: TFirebirdStmtType;
begin
    StmtHandle := nil;

    {special case for database creation -- the SQL statement must start with "CREATE DATABASE".
     The current database will not be disconnected.  the application needs to manually
     disconnect and connect to the newly created database}

    if  AnsiUpperCase(copy (trim(SQL), 1, 15)) = 'CREATE DATABASE' then
    begin
        Result := self.CreateNewDatabase(SQL);
        exit;
    end;

    try
        StatementType := PDOUtilsFirebird.PrepareStatement(self.FProtocol, @self.FHandle,
                        @self.FTrHandle, self.FDialect, SQL, StmtHandle);
        self.FProtocol.isc_dsql_execute2 (@FStatusVector, @self.FTrHandle, @StmtHandle, self.FDialect, nil, nil);
        PDOUtilsFirebird.CheckFirebirdError (self.FProtocol, self.FStatusVector, lcExecute, SQL);

        case StatementType of
            stCommit, stRollback, stUnknown: Result := -1;
        else
            begin
                Result := PDOUtilsFirebird.GetAffectedRows(self.FProtocol, StmtHandle, StatementType);
                self.FLastUpdateCount := Result;
            end;
        end;

        if self.AutoCommit then
            self.Commit;

        self.FProtocol.log(lcExecute, SQL);
    Finally
        PDOUtilsFirebird.FreeStatement(self.FProtocol, StmtHandle);
    end;
end;


function TFirebirdConnect.InitializeStatement: IPDOStatement;
begin
    Result := TPDOFirebirdStatement.create(self.FProtocol,
        self.FHandle,
        self.FTrHandle,
        self.FDialect);
end;



end.

