{ $Id: PDOConnectMySQL.pas 32 2007-01-22 01:35:20Z jmarino $

############################################################################################

	Pascal Data Objects Library
	Copyright  2006 John Marino, http://www.synsport.com
	Project site: http://pdo.sourceforge.net

############################################################################################

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

############################################################################################}


unit PDOConnectMySQL;

interface

{$I ../../directives.inc}

uses
  Types, Classes, SysUtils,
  PDOInterfaces,
  PDOConnection,
  PDOStatement,
  PDOClasses,
  PDOConstantsMySQL,
  PDOMySQLDriver,
  PDOUtilsMySQL,
  PDOLogging,
  PDOStatementMySQL,
  PDOMessages;

type
  TMySQLConnect = class(TAbstractConnection)
  private
    FProtocol: TZMySQLBaseDriver;
    FHandle: PZMySQLConnect;
    FClientCodePage: AnsiString;
    FProtocolDescription: AnsiString;




  public


    constructor Create(parameters: TStringList; protocol: TZMySQLBaseDriver);
    destructor Destroy; override;

    function InitializeStatement: IPDOStatement; override;

    procedure Commit; override;
    procedure Rollback; override;

    procedure Open; override;
    procedure Close; override;

    procedure SetTransactionIsolation(Level: TZTransactIsolationLevel); override;
    procedure SetAutoCommit(AutoCommit: Boolean); override;

    function GetProtocol: TZMySQLBaseDriver;
    function execute_without_result (SQL: AnsiString): Int64; override;
    function GetConnectionHandle: PCONNECTION; override;
    function GetSQLDialect: TSQLDialect; override;
    function GetDescription: AnsiString; override;
  protected
    function GetSilentMode: Byte; override;
    function GetCaseMode: Byte; override;
    function GetOracleNull: Byte; override;
    function GetBufferQuery: Boolean; override;
    function GetMaximumBLOBSize: Int64; override;
    function GetCompression: Boolean; override;
    function GetMultipleQuery: Boolean; override;

    procedure SetSilentMode (mode: Byte); override;
    procedure SetCaseMode (mode: Byte);   override;
    procedure SetOracleNull (mode: Byte);  override;
    procedure setBufferQuery (mode: Boolean); override;
    procedure setMaximumBLOBSize (BLOBSize: Int64); override;
    procedure setCompression (mode: Boolean); override;
    procedure setMultipleQuery (mode: Boolean); override;


    function GetLastInsertID: Int64; override;
    function GetSQLState: AnsiString; override;
    function GetLastErrorCode: Integer; override;
    function GetLastErrorMessage: AnsiString; override;
    function GetServerVersion: AnsiString; override;
    function GetClientVersion: AnsiString; override;
    function GetServerInfo: AnsiString;  override;
    function GetClientInfo: AnsiString;  override;
  end;

implementation


{**
  Constructs this object and assignes the main properties.
  @param Driver the parent PDO driver.
  @param PlainDriver a MySQL plain driver.
  @param HostName a name of the host.
  @param Port a port number (0 for default port).
  @param Database a name pof the database.
  @param User a user name.
  @param Password a user password.
  @param Info a string list with extra connection parameters.
}
constructor TMySQLConnect.Create(parameters: TStringList; protocol: TZMySQLBaseDriver);

begin
  inherited Create(
    parameters.Values['host'],
    strtoint(parameters.Values['port']),
    parameters.Values['dbname'],
    parameters.Values['username'],
    parameters.Values['password'],
    parameters.Values['unix_socket']);

  self.FProtocol := protocol;
  self.FProtocolDescription := self.FProtocol.GetProtocol;
  self.AutoCommit := true;
  TransactIsolationLevel := tiNone;

  self.Open;
end;

{**
  Destroys this object and cleanups the memory.
}
destructor TMySQLConnect.Destroy;
begin
    if not self.Closed then self.Close;
    inherited Destroy;

end;

{**
  Opens a connection to database server with specified parameters.
}
procedure TMySQLConnect.Open;
var
  LogMessage: AnsiString;
  OldLevel: TZTransactIsolationLevel;
  OldAutoCommit: Boolean;
  //ConnectTimeout: Integer;
  SQL, PHost, PSock: PChar;
  CPort: Integer;
  mysqlOptions: LongInt;
begin
  if not Closed then Exit;

  LogMessage := Format('CONNECT TO "%s" AS USER "%s"', [Database, User]);

  FProtocol.Init(FHandle);
  try
     mysqlOptions := 0;
     if self.FProtocol.GetCompression then
        mysqlOptions := mysqlOptions + _CLIENT_COMPRESS;

     if self.FProtocol.GetMultipleQuery then
        mysqlOptions := mysqlOptions + _CLIENT_MULTI_STATEMENTS;


    { Connect to MySQL database. }

    CPort := self.Port;
    PHost := PChar(self.HostName);
    PSock := nil;
    if (self.Socket <> '') then
    begin
        PSock := PChar(self.Socket);
        PHost := nil;
        CPort := 0;
    end;

    if FProtocol.RealConnect(FHandle, PHost, PChar(User),
        PChar(Password), PChar(Database), CPort, PSock, mysqlOptions) = nil then
        begin
            CheckMySQLError(FProtocol, FHandle, lcConnect, LogMessage);
            exit;
        end
      else
        self.FProtocol.log(lcConnect, LogMessage);


    { Sets a client codepage. }
    if FClientCodePage <> '' then
    begin
      SQL := PChar(Format('SET CHARACTER SET %s', [FClientCodePage]));
      if (FProtocol.ExecQuery(FHandle, SQL) <> 0) then
        begin
            CheckMySQLError(FProtocol, FHandle, lcExecute, SQL);
            exit;
        end;
     end;

    { Sets transaction isolation level. }
    OldLevel := TransactIsolationLevel;
    TransactIsolationLevel := tiNone;
    SetTransactionIsolation(OldLevel);

    { Sets an auto commit mode. }
    OldAutoCommit := AutoCommit;
    AutoCommit := True;
    SetAutoCommit(OldAutoCommit);

  except
    FProtocol.Close(FHandle);
    FProtocol.Dispose(FHandle);
    FHandle := nil;
    if (self.FProtocol.GetSilentMode = ERRORLEVEL_RAISE_EXCEPTION) then
        raise;
  end;

  inherited Open;
end;


{**
  Makes all changes made since the previous
  commit/rollback permanent and releases any database locks
  currently held by the Connection. This method should be
  used only when auto-commit mode has been disabled.

  JRM: Updated to use native function

  @see #setAutoCommit
}
procedure TMySQLConnect.Commit;
var
  SQL: PChar;
begin
  if (TransactIsolationLevel <> tiNone) and (AutoCommit <> True)
    and not Closed then
  begin
    SQL := '';
    if (FProtocol.Commit(FHandle) = false) then
        CheckMySQLError(FProtocol, FHandle, lcTransaction, 'commit: ' + SQL)
    else
        self.FProtocol.log(lcTransaction, 'commit: ' + SQL);
  end;
end;


procedure TMySQLConnect.Rollback;
begin
  if (TransactIsolationLevel <> tiNone) and (AutoCommit <> True)
    and not Closed then
  begin
    if (FProtocol.Rollback(FHandle) = false) then
        CheckMySQLError(FProtocol, FHandle, lcOther, SRollbackFailed)
    else
        self.FProtocol.log(lcOther, SRollbackSuccessful);
  end;
end;

procedure TMySQLConnect.Close;
var
  LogMessage: AnsiString;
begin
  if not Closed then
  begin
    FProtocol.Close(FHandle);
    FProtocol.Dispose(FHandle);
    {FHandle := nil;   JRM: Redundant}
    LogMessage := Format('DISCONNECT FROM "%s"', [Database]);
    self.FProtocol.log(lcDisconnect, LogMessage);
  end;
  inherited Close;
end;

{**
  Sets a new transact isolation level.
  @param Level a new transact isolation level.
}
procedure TMySQLConnect.SetTransactionIsolation(
  Level: TZTransactIsolationLevel);
var
  SQL: PChar;
  testResult: Integer;
begin
  if TransactIsolationLevel <> Level then
  begin
    inherited SetTransactionIsolation(Level);
    testResult := 1;
    if not Closed then
    begin
      case TransactIsolationLevel of
        tiNone, tiReadUncommitted:
          begin
            SQL := 'SET SESSION TRANSACTION ISOLATION LEVEL READ UNCOMMITTED';
            testResult := FProtocol.ExecQuery(FHandle, SQL);
          end;
        tiReadCommitted:
          begin
            SQL := 'SET SESSION TRANSACTION ISOLATION LEVEL READ COMMITTED';
            testResult := FProtocol.ExecQuery(FHandle, SQL);
          end;
        tiRepeatableRead:
          begin
            SQL := 'SET SESSION TRANSACTION ISOLATION LEVEL REPEATABLE READ';
            testResult := FProtocol.ExecQuery(FHandle, SQL);
          end;
        tiSerializable:
          begin
            SQL := 'SET SESSION TRANSACTION ISOLATION LEVEL SERIALIZABLE';
            testResult := FProtocol.ExecQuery(FHandle, SQL);
          end;
        else
          SQL := '';
      end;
      if (testResult <> 0) then
          CheckMySQLError(FProtocol, FHandle, lcTransaction, SQL)
      else if SQL <> '' then
          self.FProtocol.log(lcExecute, SQL);
    end;
  end;
end;

{**
  JRM: Modified this to use native functions and only set autocommit variable
  when it is successfully changed.

  @param autoCommit true enables auto-commit; false disables auto-commit.
}
procedure TMySQLConnect.SetAutoCommit(AutoCommit: Boolean);
var
  success: Boolean;
  SQL: AnsiString;
begin
  if AutoCommit <> Self.AutoCommit then
  begin
    if not Closed then
    begin
      SQL := 'Autocommit command = ';
      if (AutoCommit) then
        SQL := SQL + 'on '
      else
        SQL := SQL + 'off ';
      success := FProtocol.SetAutocommit(FHandle, AutoCommit);
      if (success) then
        begin
            self.FProtocol.log(lcExecute, format(SQuerySuccessful, [SQL]));
            inherited SetAutoCommit(AutoCommit);
        end
      else
            CheckMySQLError(FProtocol, FHandle, lcExecute, format(SQueryFailed, [SQL]));
    end;
  end;
end;

{**
  Gets a reference to MySQL connection handle.
  @return a reference to MySQL connection handle.
}
function TMySQLConnect.GetConnectionHandle: PZMySQLConnect;
begin
  Result := FHandle;
end;

{**
  Gets a MySQL plain driver interface.
  @return a MySQL plain driver interface.
}
function TMySQLConnect.GetProtocol : TZMySQLBaseDriver;
begin
  Result := FProtocol;
end;

function TMySQLConnect.execute_without_result (SQL: AnsiString): Int64;
var
    QueryHandle: PZMySQLResult;
begin
    Result := -1;
    if FProtocol.ExecQuery(FHandle, PChar(SQL)) = 0 then
      begin
        self.FProtocol.log(lcExecute, SQL);

        { Process queries with result sets }
        if FProtocol.GetStatus(FHandle) <> MYSQL_STATUS_READY then
          begin
            QueryHandle := FProtocol.StoreResult(FHandle);
            if QueryHandle <> nil then
              begin
                Result := FProtocol.GetRowCount(QueryHandle);
                FProtocol.FreeResult(QueryHandle);
              end
            else Result := FProtocol.GetAffectedRows(FHandle);
          end
         { Process regular query }
       else Result := FProtocol.GetAffectedRows(FHandle);
      end
    else CheckMySQLError(FProtocol, FHandle, lcExecute, SQL);
end;

function TMySQLConnect.InitializeStatement: IPDOStatement;
begin
    if (self.FProtocol.GetProtocol = 'mysql-5.0') then
        Result := TPDOMySqlStatement50.create(self.FProtocol, self.FHandle)
    else
        Result := TPDOMySqlStatement.create(self.FProtocol, self.FHandle);
end;
                          
function TMySQLConnect.GetSilentMode: Byte;
Begin
    Result := self.FProtocol.GetSilentMode;
End;

function TMySQLConnect.GetCaseMode: Byte;
Begin
    Result := self.FProtocol.GetCaseMode;
End;

function TMySQLConnect.GetOracleNull: Byte;
Begin
    Result := self.FProtocol.GetOracleNull;
End;

function TMySQLConnect.GetBufferQuery: Boolean;
Begin
    Result := self.FProtocol.GetBufferQuery;
End;

function TMySQLConnect.GetMaximumBLOBSize: Int64;
Begin
    Result := self.FProtocol.GetMaximumBLOBSize;
End;

function TMySQLConnect.GetSQLDialect: TSQLDialect;
Begin
    Result := self.FProtocol.GetSQLDialect;
End;

function TMySQLConnect.GetLastInsertID: Int64;
Begin
    Result := self.FProtocol.getLastInsertID(self.FHandle);
End;

function TMySQLConnect.GetSQLState: AnsiString;
Begin
    Result := self.FProtocol.getSQLState(self.FHandle);
End;

function TMySQLConnect.GetLastErrorCode: Integer;
Begin
    Result := self.FProtocol.getLastErrorCode(self.FHandle);
End;

function TMySQLConnect.GetLastErrorMessage: AnsiString;
Begin
    Result := self.FProtocol.getLastErrorMessage(self.FHandle);
End;

function TMySQLConnect.GetServerVersion: AnsiString;
Begin
    Result := self.FProtocol.GetServerVersion(self.FHandle);
End;

function TMySQLConnect.GetServerInfo: AnsiString;
Begin
    Result := self.FProtocol.GetServerInfo(self.FHandle);
End;

function TMySQLConnect.GetClientVersion: AnsiString;
Begin
    Result := self.FProtocol.GetClientVersion;
End;

function TMySQLConnect.GetClientInfo: AnsiString;
Begin
    Result := self.FProtocol.GetClientInfo;
End;

procedure TMySQLConnect.SetSilentMode(mode: Byte);
begin
    self.FProtocol.SetSilentMode(mode);
end;

procedure TMySQLConnect.SetCaseMode(mode: Byte);
begin
    self.FProtocol.SetCaseMode(mode);
end;

procedure TMySQLConnect.SetOracleNull(mode: Byte);
begin
    self.FProtocol.SetOracleNull(mode);
end;

procedure TMySQLConnect.setBufferQuery(mode: Boolean);
begin
    self.FProtocol.setBufferQuery(mode);
end;

procedure TMySQLConnect.setMaximumBLOBSize(BLOBSize: Int64);
begin
    self.FProtocol.setMaximumBLOBSize(BLOBSize);
end;

function TMySQLConnect.GetDescription: AnsiString;
begin
    Result := self.FProtocol.GetDescription;
end;

function TMySQLConnect.GetCompression: Boolean;
begin
    Result := self.FProtocol.GetCompression;
end;

procedure TMySQLConnect.setCompression(mode: Boolean);
begin
    self.FProtocol.setCompression(mode);
end;

function TMySQLConnect.GetMultipleQuery: Boolean;
begin
    Result := self.FProtocol.GetMultipleQuery;
end;

procedure TMySQLConnect.setMultipleQuery (mode: Boolean);
begin
    self.FProtocol.setMultipleQuery(mode);
end;




end.
