{ $Id: PDOUtilsMySQL.pas 23 2007-01-19 22:06:28Z jmarino $

############################################################################################

	Pascal Data Objects Library
	Copyright  2006 John Marino, http://www.synsport.com
	Project site: http://pdo.sourceforge.net

############################################################################################

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

############################################################################################}

unit PDOUtilsMySQL;

interface

{$I ../../directives.inc}

uses
  Classes, SysUtils,
  PDOSysUtils, PDOMySQLDriver, PDOLogging, 
  PDOConstantsMySQL, PDOInterfaces, PDOClasses;

const
  MAXBUF = 65535;

function ConvertMySQLHandleToSQLType(PlainDriver: TZMySQLBaseDriver;
  FieldHandle: PZMySQLField; FieldFlags: Integer): TZSQLType;
function ConvertMySQLTypeToSQLType(TypeName, TypeNameFull: string): TZSQLType;
function MySQLTimestampToDateTime(Value: string): TDateTime;
procedure CheckMySQLError(Protocol: TZMySQLBaseDriver;
  Handle: PZMySQLConnect; LogCategory: TZLoggingCategory; LogMessage: string);
procedure CheckMySQLPrepStmtError(Protocol: TZMySQLBaseDriver;
  Handle: PZMySQLConnect; LogCategory: TZLoggingCategory; LogMessage: string);


implementation

{**
  Converts a MySQL native types into PDO SQL types.
  @param PlainDriver a native MySQL plain driver.
  @param FieldHandle a handler to field description structure.
  @param FieldFlags a field flags.
  @return a SQL undepended type.
}
function ConvertMySQLHandleToSQLType(PlainDriver: TZMySQLBaseDriver;
  FieldHandle: PZMySQLField; FieldFlags: Integer): TZSQLType;
begin
  case PlainDriver.GetFieldType(FieldHandle) of
    MYSQL_TYPE_TINY:
      begin
        if (UNSIGNED_FLAG and FieldFlags) = 0 then
          Result := stByte
        else Result := stShort;
      end;
    MYSQL_TYPE_YEAR, MYSQL_TYPE_SHORT:
      begin
        if (UNSIGNED_FLAG and FieldFlags) = 0 then
          Result := stShort
        else Result := stInteger;
      end;
    MYSQL_TYPE_INT24, MYSQL_TYPE_LONG:
      begin
        if (UNSIGNED_FLAG and FieldFlags) = 0 then
          Result := stInteger
        else Result := stLong;
      end;
    MYSQL_TYPE_LONGLONG:
      begin
        if (UNSIGNED_FLAG and FieldFlags) = 0 then
          Result := stLong
        else Result := stBigDecimal;
      end;
    MYSQL_TYPE_FLOAT:
      Result := stFloat;
    MYSQL_TYPE_DECIMAL:
      begin
        if PlainDriver.GetFieldDecimals(FieldHandle) = 0 then
        begin
          if PlainDriver.GetFieldLength(FieldHandle) < 11 then
            Result := stInteger
          else Result := stLong;
        end else
          Result := stDouble;
      end;
    MYSQL_TYPE_DOUBLE:
      Result := stDouble;
    MYSQL_TYPE_DATE, MYSQL_TYPE_NEWDATE:
      Result := stDate;
    MYSQL_TYPE_TIME:
      Result := stTime;
    MYSQL_TYPE_DATETIME, MYSQL_TYPE_TIMESTAMP:
      Result := stTimestamp;
    MYSQL_TYPE_TINY_BLOB, MYSQL_TYPE_MEDIUM_BLOB,
    MYSQL_TYPE_LONG_BLOB, MYSQL_TYPE_BLOB:
      if (FieldFlags and BINARY_FLAG) = 0 then
        Result := stAsciiStream
      else Result := stBinaryStream;
    else
      Result := stString;
  end;
end;

{**
  Convert string mysql field type to SQLType
  @param string field type value
  @result the SQLType field type value
}
function ConvertMySQLTypeToSQLType(TypeName, TypeNameFull: string): TZSQLType;
var
  IsUnsigned: Boolean;
  Pos, Len: Integer;
begin
  TypeName := UpperCase(TypeName);
  TypeNameFull := UpperCase(TypeNameFull);
  Result := stUnknown;

  Pos := FirstDelimiter(' ', TypeName);
  if Pos > 0 then
    TypeName := Copy(TypeName, 1, Pos-1);

  if EndsWith(TypeNameFull, 'UNSIGNED') then
    IsUnsigned := True
  else IsUnsigned := False;

  if TypeName = 'TINYINT' then
  begin
    if IsUnsigned then
      Result := stShort
    else Result := stByte;
  end
  else if TypeName = 'YEAR' then
    Result := stShort
  else if TypeName = 'SMALLINT' then
  begin
    if IsUnsigned then
      Result := stInteger
    else Result := stShort;
  end
  else if TypeName = 'MEDIUMINT' then
    Result := stInteger
  else if TypeName = 'INT' then
  begin
    if StartsWith(TypeNameFull, 'INT(10)') or
      StartsWith(TypeNameFull, 'INT(11)') then
    begin
      if IsUnsigned then
        Result := stLong
      else Result := stInteger;
    end else
      Result := stInteger;
  end
  else if TypeName = 'INTEGER' then
  begin
    if IsUnsigned then
      Result := stLong
    else Result := stInteger;
  end
  else if TypeName = 'BIGINT' then
    Result := stLong
  else if TypeName = 'INT24' then
    Result := stLong
  else if TypeName = 'REAL' then
  begin
    if IsUnsigned then
      Result := stDouble
    else Result := stFloat;
  end
  else if TypeName = 'FLOAT' then
  begin
    if IsUnsigned then
      Result := stDouble
    else Result := stFloat;
  end
  else if TypeName = 'DECIMAL' then
  begin
    if EndsWith(TypeNameFull, ',0)') then
    begin
      Len := StrToInt(Copy(TypeNameFull, 9, Length(TypeNameFull) - 11));
      if Len < 10 then
        Result := stInteger
      else Result := stLong;
    end else
      Result := stDouble;
  end
  else if TypeName = 'DOUBLE' then
    Result := stDouble
  else if TypeName = 'CHAR' then
    Result := stString
  else if TypeName = 'VARCHAR' then
    Result := stString
  else if TypeName = 'VARBINARY' then
    Result := stBytes
  else if TypeName = 'BINARY' then
    Result := stBytes
  else if TypeName = 'DATE' then
    Result := stDate
  else if TypeName = 'TIME' then
    Result := stTime
  else if TypeName = 'TIMESTAMP' then
    Result := stTimestamp
  else if TypeName = 'DATETIME' then
    Result := stTimestamp
  else if TypeName = 'TINYBLOB' then
    Result := stBinaryStream
  else if TypeName = 'BLOB' then
    Result := stBinaryStream
  else if TypeName = 'MEDIUMBLOB' then
    Result := stBinaryStream
  else if TypeName = 'LONGBLOB' then
    Result := stBinaryStream
  else if TypeName = 'TINYTEXT' then
    Result := stAsciiStream
  else if TypeName = 'TEXT' then
    Result := stAsciiStream
  else if TypeName = 'MEDIUMTEXT' then
    Result := stAsciiStream
  else if TypeName = 'LONGTEXT' then
    Result := stAsciiStream
  else if TypeName = 'ENUM' then
  begin
    if (TypeNameFull = 'ENUM(''Y'',''N'')')
      or (TypeNameFull = 'ENUM(''N'',''Y'')') then
      Result := stBoolean
    else Result := stString;
  end
  else if TypeName = 'SET' then
    Result := stString;
end;

{**
  Converts MySQL Timestamp to TDateTime
  @param Value a timestamp string.
  @return a decoded TDateTime value.
}
function MySQLTimestampToDateTime(Value: string): TDateTime;
var
  Year, Month, Day, Hour, Min, Sec: Integer;
  StrLength, StrPos: Integer;
begin
  Month := 0;
  Day := 0;
  Hour := 0;
  Min := 0;
  Sec := 0;
  Result := 0;

  StrLength := Length(Value);
  if (StrLength = 14) or (StrLength = 8) then
  begin
    StrPos := 5;
    Year := StrToIntDef(Copy(Value, 1, 4), 0);
  end
  else
  begin
    StrPos := 3;
    Year := StrToIntDef(Copy(Value, 1, 2), 0);
  end;

  if StrLength > 2 then  {Add Month}
  begin
    Month := StrToIntDef(Copy(Value, StrPos, 2), 0);
    if StrLength > 4 then {Add Day}
    begin
      Day := StrToIntDef(Copy(Value, StrPos + 2, 2), 0);
      if StrLength > 6 then {Add Hour}
      begin
        Hour := StrToIntDef(Copy(Value, StrPos + 4, 2), 0);
        if StrLength > 8 then {Add Minute}
        begin
          Min := StrToIntDef(Copy(Value, StrPos + 6, 2), 0);
          if StrLength > 10 then {Add Second}
            Sec := StrToIntDef(Copy(Value, StrPos + 8, 2), 0);
       end;
     end;
   end;
  end;

  if (Year <> 0) and (Month <> 0) and (Day <> 0) then
  begin
    try
      Result := EncodeDate(Year, Month, Day)
    except
    end;
  end;

  try
    Result := Result + EncodeTime(Hour, Min, Sec, 0);
  except
  end;
end;

{**
  Checks for possible sql errors.
  @param PlainDriver a MySQL plain driver.
  @param Handle a MySQL connection handle.
  @param LogCategory a logging category.
  @param LogMessage a logging message.
}
procedure CheckMySQLError(Protocol: TZMySQLBaseDriver;
  Handle: PZMySQLConnect; LogCategory: TZLoggingCategory; LogMessage: string);
var
  ErrorMessage: string;
  ErrorCode: Integer;
begin
  ErrorMessage := Trim(Protocol.getLastErrorMessage(Handle));
  ErrorCode := Protocol.getLastErrorCode(Handle);

  Protocol.log(LogCategory, LogMessage, ErrorCode, ErrorMessage);
end;

procedure CheckMySQLPrepStmtError(Protocol: TZMySQLBaseDriver;
  Handle: PZMySqlPrepStmt; LogCategory: TZLoggingCategory; LogMessage: string);
var
  ErrorMessage: string;
  ErrorCode: Integer;
begin
  ErrorMessage := Trim(Protocol.GetLastPreparedError(Handle));
  ErrorCode := Protocol.GetLastPreparedErrorCode(Handle);

  Protocol.log(LogCategory, LogMessage, ErrorCode, ErrorMessage);
end;



end.
